@@ -5,10 +5,6 @@
 #
 # R TFDEA Package
 #
-# $Author: tshott $
-# $Revision: 105 $
-# $Date: 2013-08-16 22:47:28 -0700 (Fri, 16 Aug 2013) $
-#
 # Utilities File
 # General internal use only utility functions
 #
@@ -19,29 +15,43 @@
 # Add debug verbose variables - option
 # Log / export info utility - ALL info including orientation, ID string
 # * Want to make sure every run is tagged with enough info to reproduce
+# Add peers
 #
-library(lpSolveAPI)
+#library(lpSolveAPI)
 
 #
 # Package Global Variables
 #
-tfdea.version = "0.9.4"
-tfdea.id = paste0("tfdea Version ", tfdea.version)
+tfdea.version   <- "0.9.6"
+tfdea.id        <- paste0("tfdea Version ", tfdea.version)
+debug           <- 1
 
 #
 # Options Strings
 #
+
+# DEA
 options.orientation.l <- c("input","output")
-options.second.l      <- c("min", "max", "none")
+options.dual.l        <- c(FALSE, TRUE)
+options.slack.l       <- c(TRUE, FALSE)
+options.second.l      <- c("none", "min", "max")
+options.round.l       <- c(FALSE, TRUE)
+options.debug.l       <- c(1:4)
+
+# SDEA
+options.cook.l        <- c(FALSE, TRUE)
+
+# TFDEA
 options.mode.l        <- c("static", "dynamic")
 
+
 options.rts.l         <- c("vrs","drs", "crs", "irs")
-rts.rhs               <- c(  1,   1,     0,     1)
-rts.typ               <- c( "=", "<=",   0,    ">=")
+# rts.rhs               <- c(  1,   1,     0,     1)
+# rts.typ               <- c( "=", "<=",   0,    ">=")
 
 #
 # Set global epsilon for numerical checks in package
-# Backup value, dea_internal resets when run
+# Backup value, dea_internal resets when run from lp control values
 epsilon   <- 0.0003162278
 
 
@@ -57,7 +67,7 @@ epsilon   <- 0.0003162278
   if(length(dim(x)) > 2)
     stop(name," is greater then 2 dimensions", call. = FALSE)
 
-  # If data.frame - convert to array
+  # If data.frame - convert to matrix - faster to process
   if (is.data.frame(x))
     x <- data.matrix(x)
 
@@ -82,26 +92,33 @@ epsilon   <- 0.0003162278
 
 # Function .CheckDataGood
 # Check input & output data and warn about problems for DEA
-.checkDataGood <- function(x, y){
+.checkDataGood <- function(x, y, debug=1){
 
   status <- TRUE
 
+  # Make sure same number of rows
+  if (nrow(x) != nrow(y))
+    stop("Number of DMU's in inputs != number of DMU's in outputs", call. = FALSE)
+
   # Check for any zero's
   # Check for NA's
   # Check for no positive Values on input & output
   if (any(x == 0, na.rm=TRUE)){
-    cat("Warning, data has DMU's with inputs that are zero, this may cause numerical problems\n")
+    if (debug >= 1)
+      cat("Warning, data has DMU's with inputs that are zero, this may cause numerical problems\n")
     status <- FALSE
   }
 
   if (any(y == 0, na.rm=TRUE)){
-    cat("Warning, data has DMU's with outputs that are zero, this may cause numerical problems\n")
+    if (debug >= 1)
+      cat("Warning, data has DMU's with outputs that are zero, this may cause numerical problems\n")
     status <- FALSE
   }
 
   for (k in (1:nrow(x))){
     if ( all(x[k,] <= 0, na.rm=TRUE) & all(y[k,] <= 0, na.rm=TRUE)){
-      cat("Warning, DMU # ",k, "has no positive inputs or outputs, this may cause numerical problems\n")
+      if (debug >= 1)
+        cat("Warning, DMU # ",k, "has no positive inputs or outputs, this may cause numerical problems\n")
     status <- FALSE
     }
   }
@@ -128,7 +145,7 @@ epsilon   <- 0.0003162278
     if (length(dim(x)) > 2)
       stop(name," is greater then 2 dimensions", call. = FALSE)
     if (length(dim(x)) == 2 & dim(x)[2] > 1)
-      stop(name," is greater then 2 dimensions", call. = FALSE)
+      stop(name,"  must have 2nd dimension = 1", call. = FALSE)
 
     x <- as.vector(x)
   }
@@ -136,18 +153,56 @@ epsilon   <- 0.0003162278
   if (!is.numeric(x))
     stop(name, " must be numeric ", call. = FALSE)
 
-
-#   if (!is.null(ncol(x)) && (ncol(x) != 1))
-#       stop(name, " must have one col ", call. = FALSE)
-#
-
   return(x)
 }
 
 
+#
+# Check that index is OK, covert to sparse index if logical
+.checkIndex <- function(index, x, name){
+
+  if(is.null(index)){                         # NULL special case
+    return(c(1:nrow(x)))
+  }
+
+  # Is value a dataframe, array or vector?
+  if ( !is.data.frame(x) && !is.array(x) && !is.vector(x))
+    stop(name," is not a vector, array (1 dimensions) or data.frame", call. = FALSE)
+
+  if(is.data.frame(x))
+    x <- data.matrix(x)
+
+# Todo - fix how 1 dim arrays are handled
+#   if(!is.null(dim(x))){
+#     if (length(dim(x)) > 2)
+#       stop(name," is greater then 2 dimensions", call. = FALSE)
+#     if (length(dim(x)) == 2 & dim(x)[2] != 1)
+#       stop(name," must have 2nd dimension = 1", call. = FALSE)
+#
+#     x <- as.vector(x)
+#   }
+
+  if (is.logical(index)){                     # logical index - convert to sparse index
+    if (length(index) != nrow(x))
+      stop("Length ",name,"= ",length(index),"  must equal number of DMU's", call. = FALSE)
+    return(which(index))
+  }
+
+  if (is.numeric(index)){
+    if (any(index < 1) | any(index > nrow(x)))
+      stop("Value of ",name, "=", paste(index, sep=",", collapse=","),
+           " is < 1 or > number of DMU's", call.=FALSE)
+
+    if(anyDuplicated(index)){
+      warning("a value of ", name, " is duplicated", call.=TRUE)
+    }
+  }
+  return(index)
+}
+
 # Function: .checkOption()
 # Check that value in legal list options
-# If on legal list, return lowercase short form option
+# If option in legal list, return lowercase standard form of option
 #
 .checkOption <- function(value, name, options.l=c(TRUE, FALSE)){
 
@@ -155,23 +210,19 @@ epsilon   <- 0.0003162278
     stop("illegal vector for ", name, " option must be single value", call. = FALSE)
   }
 
-  # If options are character
-  if (is.character(options.l[1])){
+  if (is.character(options.l[1])){                                  # Check character
     if (is.character(value)){
       tmp.value <- tolower(value)
-      #    if (tmp.value %in% options.l)
       i <- charmatch(tmp.value, options.l, nomatch=-1)
       if (i > 0)
         return(options.l[i])
     }
-  } else if (is.logical(options.l[1])){
-    # Logical options
+  } else if (is.logical(options.l[1])){                             # Check logical
     if (is.logical(value)){
       return(value)
     }
   } else {
-    # Numeric options
-    if (is.numeric(value) && is.finite(value) && (value >= 0 )){
+    if (is.numeric(value) && is.finite(value) && (value >= 0 )){    # Numeric option
       return(value)
     }
     options.l <- c("numeric >= 0")
@@ -182,40 +233,13 @@ epsilon   <- 0.0003162278
        call. = FALSE)
 }
 
-#
-# Check that index is OK, comvert to sparse if not
-.checkIndex       <- function(index, x, name){
 
-  # Add check for are index values valid?
-
-  if(is.null(index)){                     # index.K defines the subset of DMU's that eff is
-    return(c(1:nrow(x)))
-  }
-
-  if (is.logical(index)){
-    if (length(index) != nrow(x))
-      stop("Length ",name,"= ",length(index),"  must equal number of DMU's", call. = FALSE)
-    return(which(index))                # Acccept logocal or spase index array - convert to spars
-  }
-
-  if (is.numeric(index)){
-    if (any(index < 1) | any (index > nrow(x)))
-      stop("Value of ",name, "=", paste(index, sep=",", collapse=",")," is < 1 or > number of DMU's", call.=FALSE)
-
-    if(any(duplicated(index))){
-      warning("a value of ",name, " is duplicated", call.=TRUE)
-    }
-  }
-  return(index)
-}
-
-# Check if value is weekly efficient
-# Depends on orientation
+# Check if value is weakly efficient - Depends on orientation
 # Needs to be smart FP compare
 isEfficient <- function(eff, orientation){
 
   if (!is.numeric(eff))
-    stop("Illegal value= eff; legal values are: numeric", call. = FALSE)
+    stop("Illegal value for eff; legal values are: numeric", call. = FALSE)
 
   orientation <- .checkOption(orientation,   "orientation",  options.orientation.l)
 
@@ -226,18 +250,17 @@ isEfficient <- function(eff, orientation){
   }
 }
 
-# Internal Function, check if value is efficient
-# Depends on orientation
+# Internal Function, check if value is efficient Does not depend on orientation
 # Needs to be smart FP compare
 isStdEfficient <- function(eff){
 
   # May get called with NaN value - if NaN, return false.
-  # Need to write as part of vector test
 
   return( is.finite(eff) & (eff + epsilon) >= 1)
 }
 
 
+
 #<New Page>
 # Function: .lp_debug_file()
 #
@@ -265,12 +288,74 @@ lpDebugFile <- function (lp_model, message="", file_name="lp_model.out", append=
 
   cat("\n")
   cat(paste0(message),"\n")
-  print(lp_model)
+  .print.lpModel(lp_model)
   cat("\n")
 
   sink(NULL)                              # Resets output to normal location
 }
 
+#
+# Hacked lpPrint with no limits on size
+#
+.print.lpModel <- function (x, ...){
+
+  m <- dim(x)[1]
+  n <- dim(x)[2]
+  control <- lp.control(x)
+  if (n < 1) {
+    cat(paste("Model name: ", name.lp(x), "\n", sep = ""))
+    return(invisible(x))
+  }
+#   if (n > 8) {
+#     cat(paste("Model name: ", name.lp(x), "\n", "  a linear program with ",
+#               n, " decision variables and ", m, " constraints\n",
+#               sep = ""))
+#     return(invisible(x))
+#   }
+  ans <- matrix(0, m + 1, n)
+  for (j in 1:n) {
+    col <- get.column(x, j)
+    ans[1 + col$nzrow, j] <- col$column
+  }
+  type <- get.type(x)
+  type[type == "integer"] <- "Int"
+  type[type == "real"] <- "Real"
+  kind <- get.kind(x)
+  kind[kind == "standard"] <- "Std"
+  kind[kind == "semi-continuous"] <- "S-C"
+  bounds <- get.bounds(x)
+  upper <- bounds$upper
+  lower <- bounds$lower
+  ans <- format(rbind(dimnames(x)[[2]], ans, kind, type, upper,
+                      lower), justify = "right")
+  sense <- ifelse(control$sense == "minimize", "Minimize",
+                  "Maximize")
+  lhs <- get.constr.value(x, side = "lhs")
+  rhs <- get.constr.value(x, side = "rhs")
+  rowNames <- format(c("", sense, dimnames(x)[[1]], "Kind",
+                       "Type", "Upper", "Lower"))
+  constrs <- format(c("", "", get.constr.type(x), "", "", "",
+                      ""), justify = "right")
+  rhs <- format(c("", "", as.character(rhs), "", "", "", ""),
+                justify = "right")
+  print.lhs <- any(!is.infinite(lhs[is.element(get.constr.type(x,
+                                                               as.char = FALSE), c(1, 2))]))
+  lhs <- format(c("", "", as.character(lhs), "", "", "", ""),
+                justify = "right")
+  if (print.lhs)
+    ans <- cbind(rowNames, lhs, constrs, ans, constrs, rhs)
+  else ans <- cbind(rowNames, ans, constrs, rhs)
+  ans <- apply(ans, 1, paste, collapse = "  ")
+  ans <- paste(ans, collapse = "\n")
+  model.name <- paste("Model name: ", name.lp(x), "\n", sep = "")
+  ans <- paste(model.name, ans, "\n", sep = "")
+  cat(ans)
+  invisible(x)
+
+}
+
+
+
 #
 # Utility to print the error message from lp solve for non-zero status
 #
@@ -299,6 +384,23 @@ lp_solve_error_msg <- function(status){
   return(message)
 }
 
+
+.print_status_msg <- function(status, phase, k, debug=1){
+
+  message.error   <- lp_solve_error_msg(status)
+  message.std     <- paste("Solver Phase:", phase, "Status:", status, "DMU k=", k)
+
+  if (status != 0){
+
+    if (status == 2 || status == 3){
+      if ( debug >= 2) cat(message.std, " is not in technology set: ", message.error, "\n")
+    } else {
+      if (debug >= 1) cat(message.std, " solver failed: ", message.error, "\n")
+    }
+  }
+
+}
+
 #
 # Error calculations
 #
@@ -309,3 +411,7 @@ MAD <- function(x, y){
   mad <- mean(abs(x - y), na.rm=TRUE)
   return(mad)
 }
+
+
+
+
